@@ -355,8 +355,13 @@ def raise_with_op(
             + f"\nInputs values: {scalar_values}"
         )
         if verbosity == "high":
-            detailed_err_msg += "\nInputs type_num: %s" % str(
-                [getattr(getattr(i[0], "dtype", ""), "num", "") for i in thunk.inputs]
+            detailed_err_msg += "\nInputs type_num: {}".format(
+                str(
+                    [
+                        getattr(getattr(i[0], "dtype", ""), "num", "")
+                        for i in thunk.inputs
+                    ]
+                )
             )
 
         detailed_err_msg += f"\nOutputs clients: {clients}\n"
